// Includes ////////////////////////////////////////////////////////////////

#include "UI.h"

// Constants ///////////////////////////////////////////////////////////////

const int UI::SystemTheme::MinMajor = 10;
const int UI::SystemTheme::MinMinor = 0;
const int UI::SystemTheme::MinBuild = 17763;	// 1809

const int UI::SystemTheme::DWMWA_USE_IMMERSIVE_DARK_MODE = 20;
const int UI::SystemTheme::DWMWA_USE_IMMERSIVE_DARK_MODE_BEFORE_20H1 = 19;

// Static Variables ////////////////////////////////////////////////////////

UI::SystemTheme::RtlGetNtVersionNumbersFunction UI::SystemTheme::RtlGetNtVersionNumbers = nullptr;
UI::SystemTheme::ShouldAppsUseDarkModeFunction UI::SystemTheme::ShouldAppsUseDarkMode = nullptr;
UI::SystemTheme::SetPreferredAppModeFunction UI::SystemTheme::SetPreferredAppMode = nullptr;
bool UI::SystemTheme::loaded = false;

// Functions ///////////////////////////////////////////////////////////////

UI::SystemTheme::SystemTheme()
{
	Load();
}

////////////////////////////////////////////////////////////////////////////

bool UI::SystemTheme::IsHighContrast() const
{
	HIGHCONTRASTW highContrast = {sizeof highContrast};

	if (!SystemParametersInfoW(SPI_GETHIGHCONTRAST, 0, &highContrast, 0))
		return false;

	return highContrast.dwFlags & HCF_HIGHCONTRASTON;
}

////////////////////////////////////////////////////////////////////////////

bool UI::SystemTheme::IsDarkMode() const
{
	if (!IsWindows10OrGreater())
		return false;

	if (SetPreferredAppMode)
		SetPreferredAppMode(PreferredAppMode::AllowDark);

	if (!ShouldAppsUseDarkMode)
		return false;

	if (!ShouldAppsUseDarkMode())
		return false;

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool UI::SystemTheme::SetDarkMode(const Window &window, bool enable) const
{
	if (!IsWindows10OrGreater())
		return false;

	if (SetPreferredAppMode)
		SetPreferredAppMode(enable ? PreferredAppMode::ForceDark : PreferredAppMode::ForceLight);

	int value = enable;

	if (!SUCCEEDED(DwmSetWindowAttribute(window.GetHandle(), DWMWA_USE_IMMERSIVE_DARK_MODE, &value, sizeof value)))
	if (!SUCCEEDED(DwmSetWindowAttribute(window.GetHandle(), DWMWA_USE_IMMERSIVE_DARK_MODE_BEFORE_20H1, &value, sizeof value)))
		return false;

	return true;
}

////////////////////////////////////////////////////////////////////////////

bool UI::SystemTheme::SetDarkMode(const Control &control, bool enable) const
{
	if (!IsWindows10OrGreater())
		return false;

	if (!SUCCEEDED(SetWindowTheme(control.GetHandle(), enable ? L"DarkMode_Explorer" : nullptr, nullptr)))
		return false;

	return true;
}

////////////////////////////////////////////////////////////////////////////

void UI::SystemTheme::Load() const
{
	if (loaded)
		return;

	loaded = true;

	if (!IsWindows10OrGreater())
		return;

	HMODULE ntdll = LoadLibraryW(L"ntdll.dll");

	if (!ntdll)
		return;

	RtlGetNtVersionNumbers = reinterpret_cast<RtlGetNtVersionNumbersFunction>(GetProcAddress(ntdll, "RtlGetNtVersionNumbers"));

	if (!RtlGetNtVersionNumbers)
		return;

	DWORD major;
	DWORD minor;
	DWORD build;

	RtlGetNtVersionNumbers(&major, &minor, &build);
	build &= 0xFFFFFFF;

	if (major == MinMajor && minor == MinMinor && build >= MinBuild || major == MinMajor && minor > MinMinor || major > MinMajor)
	{
		HMODULE uxtheme = LoadLibraryW(L"uxtheme.dll");

		if (!uxtheme)
			return;

		ShouldAppsUseDarkMode = reinterpret_cast<ShouldAppsUseDarkModeFunction>(GetProcAddress(uxtheme, MAKEINTRESOURCEA(132)));
		SetPreferredAppMode = reinterpret_cast<SetPreferredAppModeFunction>(GetProcAddress(uxtheme, MAKEINTRESOURCEA(135)));
	}
}

////////////////////////////////////////////////////////////////////////////
